@@ -56,7 +56,7 @@ class FactoryEntry(object):
         self.factory = factory
         self.streams = {}
 
-    def __repr__(self):
+    def __str__(self):
         return "<FactoryEntry %s>" % self.factory
 
 class StreamEntry(object):
@@ -81,7 +81,7 @@ class StreamEntry(object):
 
         return entry
 
-    def __repr__(self):
+    def __str__(self):
         return "<StreamEntry %s '%s'>" % (self.factory_entry.factory, self.stream)
 
 class Pipeline(Signallable, Loggable):
@@ -413,6 +413,7 @@ class Pipeline(Signallable, Loggable):
                                   gst.SEEK_TYPE_SET, position,
                                   gst.SEEK_TYPE_NONE, -1)
         if not res:
+            self.debug("seeking failed")
             raise PipelineError("seek failed")
         self.debug("seeking succesfull")
         self.emit('position', position)
