@@ -105,6 +105,14 @@ GlobalSettings.addConfigOption('lastExportFolder',
                                 key="last-export-folder",
                                 environment="PITIVI_EXPORT_FOLDER",
                                 default=os.path.expanduser("~"))
+GlobalSettings.addConfigOption('elementSettingsDialogWidth',
+    section='export',
+    key='element-settings-dialog-width',
+    default = 620)
+GlobalSettings.addConfigOption('elementSettingsDialogHeight',
+    section='export',
+    key='element-settings-dialog-height',
+    default = 460)
 
 def supported(info):
     return formatter.can_handle_location(info[1])
@@ -476,10 +484,10 @@ class PitiviMainWindow(gtk.Window, Loggable):
             self.settings.mainWindowHeight = event.height
 
     def _deleteCb(self, unused_widget, unused_data=None):
+        self._saveWindowSettings()
         if not self.app.shutdown():
             return True
 
-        self._saveWindowSettings()
         return False
 
     def _exposeEventCb(self, unused_widget, event):
@@ -789,6 +797,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         dialog.get_content_area().pack_start(chooser, True, True)
         chooser.show()
 
+        dialog.set_size_request(640, 480)
         response = dialog.run()
 
         if response == gtk.RESPONSE_OK:
@@ -830,11 +839,6 @@ class PitiviMainWindow(gtk.Window, Loggable):
         undo_action = self.actiongroup.get_action("Undo")
         can_undo = bool(action_log.undo_stacks)
         undo_action.set_sensitive(can_undo)
-        if can_undo:
-            stack = action_log.undo_stacks[-1]
-            undo_action.props.label = _("Undo %s") % stack.action_group_name
-        else:
-            undo_action.props.label = _("Undo")
 
         dirty = action_log.dirty()
         save_action = self.actiongroup.get_action("SaveProject")
@@ -844,11 +848,6 @@ class PitiviMainWindow(gtk.Window, Loggable):
         redo_action = self.actiongroup.get_action("Redo")
         can_redo = bool(action_log.redo_stacks)
         redo_action.set_sensitive(can_redo)
-        if can_redo:
-            stack = action_log.redo_stacks[-1]
-            redo_action.props.label = _("Redo %s") % stack.action_group_name
-        else:
-            redo_action.props.label = _("Redo")
 
 ## PiTiVi current project callbacks
 
@@ -906,6 +905,10 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.timeline.timelinePositionChanged(position)
         self.timelinepos = position
 
+    @handler(project_pipeline, "state-changed")
+    def _timelinePipelineStateChangedCb(self, pipeline, state):
+        self.timeline.stateChanged(state)
+
 ## Project Timeline (not to be confused with UI timeline)
 
     project_timeline = receiver()
