@@ -24,6 +24,7 @@ Timeline widgets for the complex view
 """
 
 import gtk
+import urllib
 
 from pitivi.log.loggable import Loggable
 import ruler
@@ -36,6 +37,8 @@ from timelinecanvas import TimelineCanvas
 from timelinecontrols import TimelineControls
 from pitivi.receiver import receiver, handler
 from zoominterface import Zoomable
+from pitivi.ui.common import LAYER_HEIGHT_EXPANDED, LAYER_SPACING
+from pitivi.timeline.timeline import MoveContext
 
 # tooltip text for toolbar
 DELETE = _("Delete Selected")
@@ -128,6 +131,7 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self._factories = None
         self._finish_drag = False
         self._position = 0
+        self._state = gst.STATE_NULL
         self._createUI()
         self._prev_duration = 0
         self.shrink = True
@@ -139,11 +143,10 @@ class Timeline(gtk.Table, Loggable, Zoomable):
 
         # controls for tracks and layers
         self._controls = TimelineControls()
-        self._controls.connect('track-expanded',
-                self._timelineControlsTrackExpandedCb)
         controlwindow = gtk.ScrolledWindow(None, self.vadj)
         controlwindow.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
         controlwindow.add_with_viewport(self._controls)
+        controlwindow.set_size_request(-1, 1)
         self.attach(controlwindow, 0, 1, 1, 2, xoptions=0)
 
         # timeline ruler
@@ -227,10 +230,6 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self._canvas.connect("button-release-event", self._buttonRelease)
         self._canvas.connect("key-press-event", self._keyPressEventCb)
 
-    def _timelineControlsTrackExpandedCb(self, timeline_controls,
-            track, expanded):
-        self._canvas.setExpanded(track, expanded)
-
 
 ## Event callbacks
 
@@ -276,7 +275,7 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self.warning("self._factories:%r, self._temp_objects:%r",
                      not not self._factories,
                      not not self._temp_objects)
-        if not self._factories:
+        if self._factories is None:
             atom = gtk.gdk.atom_intern(dnd.FILESOURCE_TUPLE[0])
             self.drag_get_data(context, atom, timestamp)
             self.drag_highlight()
@@ -285,6 +284,9 @@ class Timeline(gtk.Table, Loggable, Zoomable):
             if not self._temp_objects:
                 self.timeline.disableUpdates()
                 self._add_temp_source()
+                focus = self._temp_objects[0]
+                self._move_context = MoveContext(self.timeline,
+                        focus, set(self._temp_objects[1:]))
             self._move_temp_source(x, y)
         return True
 
@@ -299,8 +301,14 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self.timeline.enableUpdates()
 
     def _dragDropCb(self, widget, context, x, y, timestamp):
+        self.app.action_log.begin("add clip")
         self._add_temp_source()
+        focus = self._temp_objects[0]
+        self._move_context = MoveContext(self.timeline,
+                focus, set(self._temp_objects[1:]))
         self._move_temp_source(x, y)
+        self._move_context.finish()
+        self.app.action_log.commit()
         context.drop_finish(True, timestamp)
         self._factories = None
         self._temp_objects = None
@@ -321,7 +329,7 @@ class Timeline(gtk.Table, Loggable, Zoomable):
             uris = selection.data.split("\n")
         else:
             context.finish(False, False, timestamp)
-        self._factories = [self.project.sources[uri] for uri in uris]
+        self._factories = [self.project.sources.getUri(uri) for uri in uris]
         context.drag_status(gtk.gdk.ACTION_COPY, timestamp)
         return True
 
@@ -333,11 +341,9 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         x1, y1, x2, y2 = self._controls.get_allocation()
         offset = 10 + (x2 - x1)
         x, y = self._canvas.convert_from_pixels(x - offset, y)
+        priority = int((y // (LAYER_HEIGHT_EXPANDED + LAYER_SPACING)))
         delta = Zoomable.pixelToNs(x)
-        for obj in self._temp_objects:
-            obj.setStart(max(0, delta), snap=True)
-            delta += obj.duration
-
+        self._move_context.editTo(delta, priority)
 
 ## Zooming and Scrolling
 
@@ -352,6 +358,9 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self._canvas._position = position
         self.scrollToPlayhead()
 
+    def stateChanged(self, state):
+        self._state = state
+
     def scrollToPlayhead(self):
         width = self.get_allocation().width
         new_pos = Zoomable.nsToPixel(self._position)
@@ -361,6 +370,8 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         return False
 
     def scrollToPosition(self, position):
+        if not self._state == gst.STATE_PLAYING:
+            return
         if position > self.hadj.upper:
             # we can't perform the scroll because the canvas needs to be
             # updated
@@ -469,7 +480,9 @@ class Timeline(gtk.Table, Loggable, Zoomable):
 
     def deleteSelected(self, unused_action):
         if self.timeline:
+            self.app.action_log.begin("delete clip")
             self.timeline.deleteSelection()
+            self.app.action_log.commit()
 
     def unlinkSelected(self, unused_action):
         if self.timeline:
